"use strict";exports.__esModule = true;exports.parseArgs = parseArgs;exports.parseExpression = parseExpression;exports.parseParams = parseParams;exports.parseStatements = parseStatements;exports.parseTemplateLiteral = parseTemplateLiteral;exports.parseVar = parseVar;var babelParser = _interopRequireWildcard(require("@babel/parser"));
var _loc = require("./loc");
var _compiler = require("@marko/compiler");function _getRequireWildcardCache(nodeInterop) {if (typeof WeakMap !== "function") return null;var cacheBabelInterop = new WeakMap();var cacheNodeInterop = new WeakMap();return (_getRequireWildcardCache = function (nodeInterop) {return nodeInterop ? cacheNodeInterop : cacheBabelInterop;})(nodeInterop);}function _interopRequireWildcard(obj, nodeInterop) {if (!nodeInterop && obj && obj.__esModule) {return obj;}if (obj === null || typeof obj !== "object" && typeof obj !== "function") {return { default: obj };}var cache = _getRequireWildcardCache(nodeInterop);if (cache && cache.has(obj)) {return cache.get(obj);}var newObj = {};var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;for (var key in obj) {if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;if (desc && (desc.get || desc.set)) {Object.defineProperty(newObj, key, desc);} else {newObj[key] = obj[key];}}}newObj.default = obj;if (cache) {cache.set(obj, newObj);}return newObj;}

const CODE_AS_WHITE_SPACE_KEY = Symbol();

function parseStatements(
file,
str,
sourceStart,
sourceEnd,
sourceOffset)
{
  return tryParse(file, false, str, sourceStart, sourceEnd, sourceOffset);
}

function parseExpression(
file,
str,
sourceStart,
sourceEnd,
sourceOffset)
{
  return tryParse(file, true, str, sourceStart, sourceEnd, sourceOffset);
}

function parseParams(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
  file,
  `(${str})=>{}`,
  sourceStart,
  sourceEnd,
  1);


  if (parsed.type === "ArrowFunctionExpression") {
    return parsed.params;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

function parseArgs(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(file, `_(${str})`, sourceStart, sourceEnd, 2);

  if (parsed.type === "CallExpression") {
    return parsed.arguments;
  }

  return [ensureParseError(file, parsed, sourceStart, sourceEnd)];
}

function parseVar(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
  file,
  `(${str})=>{}`,
  sourceStart,
  sourceEnd,
  1);


  if (parsed.type === "ArrowFunctionExpression" && parsed.params.length === 1) {
    return parsed.params[0];
  }

  return ensureParseError(file, parsed, sourceStart, sourceEnd);
}

function parseTemplateLiteral(file, str, sourceStart, sourceEnd) {
  const parsed = parseExpression(
  file,
  "`" + str + "`",
  sourceStart,
  sourceEnd,
  1);


  if (parsed.type === "TemplateLiteral") {
    return parsed;
  }

  return ensureParseError(file, parsed, sourceStart, sourceEnd);
}

function tryParse(
file,
isExpression,
str,
sourceStart,
sourceEnd,
sourceOffset)
{
  const { parserOpts } = file.opts;
  let code = str;

  if (typeof sourceStart === "number") {
    const whitespace =
    file.metadata.marko[CODE_AS_WHITE_SPACE_KEY] || (
    file.metadata.marko[CODE_AS_WHITE_SPACE_KEY] = file.code.replace(
    /[^\s]/g,
    " "));

    code =
    whitespace.slice(
    0,
    sourceOffset ? sourceStart - sourceOffset : sourceStart) +
    str;

    try {
      return isExpression ?
      babelParser.parseExpression(code, parserOpts) :
      babelParser.parse(code, parserOpts).program.body;
    } catch (err) {
      const parseError = createParseError(
      file,
      sourceStart,
      sourceEnd,
      err.message,
      err.loc);


      if (isExpression) {
        return parseError;
      } else {
        return [parseError];
      }
    }
  } else {
    return isExpression ?
    _compiler.types.cloneDeepWithoutLoc(babelParser.parseExpression(code, parserOpts)) :
    babelParser.
    parse(code, parserOpts).
    program.body.map((node) => _compiler.types.cloneDeepWithoutLoc(node));
  }
}

function ensureParseError(file, node, sourceStart, sourceEnd) {
  if (node.type === "MarkoParseError") return node;
  return createParseError(
  file,
  sourceStart,
  sourceEnd,
  `Unexpected node of type ${node.type} returned while parsing.`);

}

function createParseError(file, sourceStart, sourceEnd, label, errorLoc) {
  file.___hasParseErrors = true;
  const loc = (0, _loc.getLocRange)(file, sourceStart, sourceEnd);
  return {
    type: "MarkoParseError",
    source: file.code.slice(sourceStart, sourceEnd),
    label: label.replace(/ *\(\d+:\d+\)$/, ""),
    errorLoc: errorLoc && getBoundedRange(loc, errorLoc),
    loc,
    start: sourceStart,
    end: sourceEnd
  };
}

function getBoundedRange(sourceRange, start) {
  if (start && typeof start.index === "number") {
    if (
    start.index < sourceRange.start.index ||
    start.index >= sourceRange.end.index)
    {
      return sourceRange;
    }

    return { start, end: start };
  }
}